//
//  CAModel.swift
//  Calculator
//
//  Created by narita on 2017/08/26.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

class CAModel : NSObject, NSUserInterfaceValidations
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    override init()
    {
        self.value = 0
        self.operation = .None
        self.display = ""
    }
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods

    // クリア
    @IBAction func clear(_ sender: NSObject)
    {
        self.value = 0
        self.operation = .None
        self.display = ""
    }
    
    @objc func canClear(_ sender: NSObject) -> Bool
    {
        return true != ( self.value == 0 && self.operation == .None && self.display == "" )
    }

    // 数字入力
    @IBAction func digit(_ sender: NSObject)
    {
        if let theControl = sender as? NSControl
        {
            let theTag = theControl.tag
            
            // ~=はパターンマッチ演算子。0...9のパターンに、theTagがマッチするかを調べる。
            // 結果的に、theTagが0〜9の値であればtrue
            if 0...9 ~= theTag
            {
                self.display += String.init(format: "%d", theTag)
            }
        }
    }

    @objc func canDigit(_ sender: NSObject) -> Bool
    {
        return true
    }

    // 小数点
    @IBAction func dot(_ sender: NSObject)
    {
        // "."が含まれている文字列にさらに"."を追加しては駄目
        guard (self.display.range(of: ".") == nil) else
        {
            return
        }
        
        self.display += "."
    }

    @objc func canDot(_ sender: NSObject) -> Bool
    {
        return self.display.range(of: ".") == nil
    }

    // +/-の反転
    @IBAction func sign(_ sender: NSObject)
    {
        if let theDisplayValue = Double(self.display)
        {
            switch theDisplayValue
            {
                // "-"記号を取り除く
                case let v where v < 0:
                    self.display = String(theDisplayValue)
                    
                    break

                // "-"記号を先頭に追加する
                case let v where v > 0:
                    self.display = "-" + self.display
                    break

                default:
                    break
            }
        }
    }

    @objc func canSign(_ sender: NSObject) -> Bool
    {
        return self.display != "" && self.display != "0"
    }

    // = 演算
    @IBAction func equal(_ sender: NSObject)
    {
        var theResult: Double = 0.0
        
        switch self.operation
        {
            case .None:
                break
                
            case .Plus:
                theResult = self.value + Double(self.display)!

            case .Minus:
                theResult = self.value - Double(self.display)!

            case .Time:
                theResult = self.value * Double(self.display)!

            case .Divide:
                theResult = self.value / Double(self.display)!
            
        }
    
        self.display = String(theResult)
    }

    @objc func canEqual(_ sender: NSObject) -> Bool
    {
        return self.operation != .None && self.display != ""
    }
    
    // + 演算
    @IBAction func operaterPlus(_ sender: NSObject)
    {
        self.value = Double(self.display)!
        self.operation = .Plus
        self.display = ""
    }
    
    @objc func canOperaterPlus(_ sender: NSObject) -> Bool
    {
        return self.operation == .None && self.display != ""
    }

    // - 演算
    @IBAction func operaterMinus(_ sender: NSObject)
    {
        self.value = Double(self.display)!
        self.operation = .Minus
        self.display = ""
    }

    @objc func canOperaterMinus(_ sender: NSObject) -> Bool
    {
        return self.operation == .None && self.display != ""
    }

    // * 演算
    @IBAction func operaterTime(_ sender: NSObject)
    {
        self.value = Double(self.display)!
        self.operation = .Time
        self.display = ""
    }

    @objc func canOperaterTime(_ sender: NSObject) -> Bool
    {
        return self.operation == .None && self.display != ""
    }

    // / 演算
    @IBAction func operaterDivide(_ sender: NSObject)
    {
        self.value = Double(self.display)!
        self.operation = .Divide
        self.display = ""
    }

    @objc func canOperaterDivide(_ sender: NSObject) -> Bool
    {
        return self.operation == .None && self.display != ""
    }

    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    var value   : Double = 0.0
    @objc dynamic var display : String
    
    enum CalcOperationType
    {
        case None
        case Plus
        case Minus
        case Time
        case Divide
    }
    
    var operation: CalcOperationType = .None
 
    // MARK: - Utility methods
    
    func validateUserInterfaceItem(_ item: NSValidatedUserInterfaceItem) -> Bool
    {
        return respondsAndPerformValidatedUserInterface(item, nil)
    }
}
