//
//  CCAppDelegate.swift
//  CurrencyConverter
//
//  Created by narita on 2017/08/27.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

@NSApplicationMain
class CCAppDelegate: NSObject, NSApplicationDelegate, NSUserInterfaceValidations, NSWindowRestoration
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    @objc static func restoreWindow(withIdentifier identifier: NSUserInterfaceItemIdentifier,
                              state: NSCoder,
                              completionHandler: @escaping (NSWindow?, Error?) -> Void)
    {
        var theWindow: NSWindow? = nil

        if identifier == NSUserInterfaceItemIdentifier.init(rawValue:"MINTLWindow")
        {
            let appDelegate = NSApp.delegate as! CCAppDelegate
            
            theWindow = appDelegate.makeNewWindow()!.window
        }
        
        completionHandler(theWindow, nil)
    }

    
    // MARK: - life cycle methods

    // MARK: - action methods
    
    @IBAction func newWindow(_ sender: NSObject!)
    {
        _ = self.makeNewWindow()
    }
    
    @objc func canNewWindow(_ sender: NSObject!) -> Bool
    {
        return true
    }

    func makeNewWindow() -> NSWindowController?
    {
        if let theWindowController =
            NSStoryboard.init(name: NSStoryboard.Name.init(rawValue:"CCWindowController"),
                              bundle:nil).instantiateInitialController() as? NSWindowController
        {
            theWindowController.contentViewController!.representedObject = self.model
            
            theWindowController.window?.isRestorable = true
            theWindowController.window?.restorationClass = CCAppDelegate.self
            theWindowController.window?.identifier = NSUserInterfaceItemIdentifier(rawValue: "MINTLWindow")
            
            theWindowController.showWindow(nil)
            
            NotificationCenter.default.addObserver(self,
                                                   selector: NSSelectorFromString("willCloseWindow:"),
                                                   name: NSWindow.willCloseNotification,
                                                   object: theWindowController.window)
            
            self.windowControllers.insert(theWindowController)
            
            return theWindowController
        }
        
        return nil
    }

    // MARK: - event handling methods
    
    @objc func willCloseWindow(_ aNotification: Notification)
    {
        if let theWindow = aNotification.object as! NSWindow!
        {
            self.windowControllers.remove(theWindow.windowController!)
        }
    }
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    func applicationDidFinishLaunching(_ aNotification: Notification)
    {
        // Insert code here to initialize your application
        
        if self.windowControllers.isEmpty
        {
            self.newWindow(nil)
        }
    }
    
    func applicationWillTerminate(_ aNotification: Notification)
    {
        // Insert code here to tear down your application
    }
    
    // このメソッドがYESを返す事で最後のWindowがクローズされた時にアプリケーションが終了します
    func applicationShouldTerminateAfterLastWindowClosed(_ anApplication: NSApplication) -> Bool
    {
        return true
    }
    
    // MARK: - accessor methods (in pairs)
    
    @objc lazy var model: CCModel? =
    {
        return CCModel.init()
    }()

    var windowControllers: Set<NSWindowController> = Set.init()

    
    // MARK: - Utility methods

    func validateUserInterfaceItem(_ item: NSValidatedUserInterfaceItem) -> Bool
    {
        return respondsAndPerformValidatedUserInterface(item)
    }
}

