//
//  TAAppDelegate.swift
//  TravelAdvisor
//
//  Created by narita on 2017/08/29.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

@NSApplicationMain
class TAAppDelegate: NSObject, NSApplicationDelegate, NSUserInterfaceValidations, NSWindowRestoration
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    @objc static func restoreWindow(withIdentifier identifier: NSUserInterfaceItemIdentifier,
                                    state: NSCoder,
                                    completionHandler: @escaping (NSWindow?, Error?) -> Void)
    {
        var theWindow: NSWindow? = nil
        
        if identifier == NSUserInterfaceItemIdentifier.init(rawValue:"TAWindow")
        {
            let appDelegate = NSApp.delegate as! TAAppDelegate
            
            theWindow = appDelegate.makeNewWindow()!.window
        }
        
        completionHandler(theWindow, nil)
    }

    // MARK: - life cycle methods
    
    // MARK: - action methods

    @IBAction func newWindow(_ sender: NSObject!)
    {
        _ = self.makeNewWindow()
    }
    
    @objc func canNewWindow(_ sender: NSObject!) -> Bool
    {
        // もしもWindowを2つ以上表示したくないならば、
        // return self.windowControllers.count == 0  として、windowがない時だけ、表示可能にすると良い
        
        return true
    }
    
    func makeNewWindow() -> TAWindowController?
    {
        if let theWindowController =
            NSStoryboard.init(name: NSStoryboard.Name.init(rawValue:"TAWindowController"),
                              bundle:nil).instantiateInitialController() as? TAWindowController
        {
            theWindowController.model = self.model
            
            theWindowController.window?.isRestorable = true
            theWindowController.window?.restorationClass = TAAppDelegate.self
            theWindowController.window?.identifier = NSUserInterfaceItemIdentifier(rawValue: "TAWindow")
            
            NotificationCenter.default.addObserver(self,
                                                   selector: NSSelectorFromString("willCloseWindow:"),
                                                   name: NSWindow.willCloseNotification,
                                                   object: theWindowController.window)
            
            self.windowControllers.insert(theWindowController)
            
            theWindowController.showWindow(nil)
            
            return theWindowController
        }
        
        return nil
    }

    @IBAction func saveAction(_ sender: NSObject?)
    {
        _ = self.model!.save()
    }

    @objc func canSaveAction(_ sender: NSObject!) -> Bool
    {
        return self.model!.hasChanges()
    }

    // MARK: - event handling methods

    @objc func willCloseWindow(_ aNotification: Notification)
    {
        if let theWindow = aNotification.object as! NSWindow!
        {
            self.windowControllers.remove(theWindow.windowController!)
        }
    }

    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods

    func applicationDidFinishLaunching(_ aNotification: Notification)
    {
        if self.windowControllers.isEmpty
        {
            self.newWindow(nil)
        }
    }

    func applicationShouldTerminate(_ sender: NSApplication) -> NSApplication.TerminateReply
    {
        var theResult: NSApplication.TerminateReply = .terminateNow
        
        switch self.model!.save()
        {
            case .none:
                // ここに来ることはあり得ないが、来た場合は終了
                theResult = .terminateNow

            case .commitFail:
                theResult = .terminateCancel
            
            case .saveFail:
                do
                {
                    let question = NSLocalizedString("Could not save changes while quitting. Quit anyway?",
                                                     comment: "Quit without saves error question message")
                    let info = NSLocalizedString("Quitting now will lose any changes you have made since the last successful save",
                                                 comment: "Quit without saves error question info");
                    let quitButton = NSLocalizedString("Quit anyway",
                                                       comment: "Quit anyway button title")
                    let cancelButton = NSLocalizedString("Cancel",
                                                         comment: "Cancel button title")
                    let alert = NSAlert()
                    alert.messageText = question
                    alert.informativeText = info
                    alert.addButton(withTitle: quitButton)
                    alert.addButton(withTitle: cancelButton)
                    
                    let answer = alert.runModal()
                    if answer == .alertSecondButtonReturn
                    {
                        theResult = .terminateCancel
                    }
                }
            
            case .saveFailAndPresentError:
                // メッセージは表示済みなので、単純にキャンセルだけ
                theResult = .terminateCancel

            case .saveSuccess:
                theResult = .terminateNow
        }
        
        return theResult
    }

    func applicationWillTerminate(_ aNotification: Notification)
    {

    }

    // このメソッドがYESを返す事で最後のWindowがクローズされた時にアプリケーションが終了します
    func applicationShouldTerminateAfterLastWindowClosed(_ anApplication: NSApplication) -> Bool
    {
        return true
    }

    // MARK: - accessor methods (in pairs)

    @objc lazy var model: TAModel? =
    {
        return TAModel.init()
    }()

    @objc var windowControllers: Set<NSWindowController> = Set.init()
    
    // MARK: - Utility methods

    func validateUserInterfaceItem(_ item: NSValidatedUserInterfaceItem) -> Bool
    {
        return respondsAndPerformValidatedUserInterface(item)
    }
}

