//
//  TAModel.swift
//  TravelAdvisor
//
//  Created by narita on 2017/08/30.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

class TAModel: NSObject
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    public enum saveResultType
    {
        case none                       // 状態不明
        case commitFail                 // 編集結果のコミットに失敗した
        case saveFail                   // 保存に失敗した
        case saveFailAndPresentError    // 保存に失敗し、アプリケーションはエラーメッセージを通知した
        case saveSuccess                // 保存に成功した
    }
    
    func save() -> saveResultType
    {
        // Performs the save action for the application, which is to send the save: message to the application's managed object context. Any encountered errors are presented to the user.
        let context = persistentContainer.viewContext
        
        if !context.commitEditing()
        {
            NSLog("\(NSStringFromClass(type(of: self))) unable to commit editing before saving")
            
            return .commitFail
        }
        
        if context.hasChanges
        {
            do
            {
                try context.save()
            }
            catch
            {
                // Customize this code block to include application-specific recovery steps.
                let nserror = error as NSError
                let result  = NSApplication.shared.presentError(nserror)
                
                if (result)
                {
                    return .saveFailAndPresentError
                }

                return .saveFail
            }
        }
        
        return .saveSuccess
    }

    func hasChanges() -> Bool
    {
        return self.persistentContainer.viewContext.hasChanges
    }
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @objc var managedObjectContext: NSManagedObjectContext
    {
        return self.persistentContainer.viewContext
    }
    
    @objc lazy var persistentContainer: NSPersistentContainer =
    {
            /*
             The persistent container for the application. This implementation
             creates and returns a container, having loaded the store for the
             application to it. This property is optional since there are legitimate
             error conditions that could cause the creation of the store to fail.
             */
            let container = NSPersistentContainer(name: "TravelAdvisor")
            container.loadPersistentStores(completionHandler:
            {
                (storeDescription, error) in
                
                if let error = error
                {
                    // Replace this implementation with code to handle the error appropriately.
                    // fatalError() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
                    
                    /*
                     Typical reasons for an error here include:
                     * The parent directory does not exist, cannot be created, or disallows writing.
                     * The persistent store is not accessible, due to permissions or data protection when the device is locked.
                     * The device is out of space.
                     * The store could not be migrated to the current model version.
                     Check the error message to determine what the actual problem was.
                     */
                    fatalError("Unresolved error \(error)")
                }
                
                // メモ： "What's New in Core Data in macOS 10.12, iOS 10.0, tvOS 10.0, and watchOS 3.0"に
                // "NSManagedObjectContext now defaults to a nil NSUndoManager on macOS. It always defaulted to nil on iOS."と
                // 衝撃的な事が書かれている。自分でundoManagerをセットする必要がある
                container.viewContext.undoManager = UndoManager.init()
            })
            return container
    }()

    // MARK: - Utility methods
}
