//
//  MINTLMenu.swift
//  MenuTester
//
//  Created by narita on 2017/08/14.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa
import Carbon

class MINTLMenu: NSMenu
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    deinit
    {
        self.disableCustomDraw()
    }

    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    func enableCustomDraw()
    {
        if( self.eventHandlerRef == nil )
        {
            let events = UnsafeMutablePointer<EventTypeSpec>.allocate( capacity: 2 )
            events[0] = EventTypeSpec( eventClass: OSType(kEventClassMenu),
                                       eventKind:  UInt32(kEventMenuDrawItem) )
            events[1] = EventTypeSpec( eventClass: OSType(kEventClassMenu),
                                       eventKind:  UInt32(kEventMenuMeasureItemWidth) )
            
            // 自分自身のアドレスをswift風のポインターにする
            let selfPtr = Unmanaged.passUnretained(self).toOpaque()
            
            // この関数内関数で、メソッドへ転送する。C++でよくやっていたWindowPtrのuserInfoにオブジェクトのポインターを仕込む
            func  menuEventHandler( _ inHandlerRef:EventHandlerCallRef?,
                                    _ inEvent:EventRef?,
                                    _ inUserInfo:UnsafeMutableRawPointer? ) -> OSStatus
            {
                var theResult: OSStatus = 0;
                
                // swift風味のポインターを元に戻してObjectとして扱う
                let theNSMenu: MINTLMenu = Unmanaged<MINTLMenu>.fromOpaque(inUserInfo!).takeUnretainedValue()
                
                theResult = theNSMenu.carbonMenu(eventHandler: inHandlerRef,
                                                 event: inEvent)
                
                return theResult;
            }

            if self.menuRef != nil
            {
                let theMenuRef: Menu = self.menuRef!
                
                InstallEventHandler( self.getMenuEventTarget(theMenuRef),
                                     menuEventHandler as EventHandlerUPP,
                                     2,
                                     events,
                                     selfPtr,
                                     &eventHandlerRef)
                
                // indexを走査してMINTLMenuItemの物をChangeMenuItemAttributesをONにする
                for (index, item) in self.items.enumerated()
                {
                    if let mintItem: MINTLMenuItem = item as? MINTLMenuItem
                    {
                        var theResult: OSStatus = 0

                        theResult = self.changeMenuItemAttributes(theMenuRef, UInt16(index) + 1, UInt32(kMenuItemAttrCustomDraw), 0)
                        
                        if theResult == 0
                        {
                            mintItem.keyEquivalent = ""
                            mintItem.keyEquivalentModifierMask = NSEvent.ModifierFlags(rawValue:0)
                        }
                    }
                }
            }
        }
    }

    func disableCustomDraw()
    {
        // keyEquivalentやkeyEquivalentModifierMaskをクリアしてしまているので、実際には全ては復帰しない。
        if( eventHandlerRef != nil )
        {
            RemoveEventHandler(eventHandlerRef)
            eventHandlerRef = nil
        }
    }

    static func switchCustomDraw(_ inBool: Bool, _ inMenu: NSMenu)
    {
        if let theMenu = inMenu as? MINTLMenu
        {
            if( inBool )
            {
                theMenu.enableCustomDraw()
            }
            else
            {
                theMenu.disableCustomDraw()
            }
        }
        
        for i in inMenu.items
        {
            // subMenuを持っている場合には掘ってゆく
            if( i.hasSubmenu )
            {
                MINTLMenu.switchCustomDraw(inBool, i.submenu!)
            }
        }
    }

    // 再帰的にカスタム描画をenableにする
    static func enableCustomDrawOnMainMenu()
    {
        MINTLMenu.switchCustomDraw(true, NSApp.mainMenu!)
    }

    static func disableCustomDrawOnMainMenu()
    {
        MINTLMenu.switchCustomDraw(false, NSApp.mainMenu!)
    }

    // MARK: - event handling methods
    
    func carbonMenu(eventHandler inHandlerRef: EventHandlerCallRef?,
                    event inEvent: EventRef?) -> OSStatus
    {
        var theResult: OSStatus = 0

        if( GetEventClass(inEvent) == OSType(kEventClassMenu) )
        {
            theResult = CallNextEventHandler(inHandlerRef, inEvent)

            var  theTrackingData: MenuTrackingData = MenuTrackingData.init()
            GetMenuTrackingData(self.menuRef, &theTrackingData);

            var theItemIndex: MenuItemIndex = 0
            GetEventParameter(inEvent,
                              EventParamName(kEventParamMenuItemIndex),
                              EventParamType(typeMenuItemIndex),
                              nil,
                              MemoryLayout<MenuItemIndex>.size,
                              nil,
                              &theItemIndex)

            let theMenuItem: NSMenuItem? = self.item(at:Int(theItemIndex) - 1)
            let theItem: MINTLMenuItem?  = theMenuItem as! MINTLMenuItem?
            
            if( theMenuItem is MINTLMenuItem )
            {
                if( GetEventKind(inEvent) == OSType(kEventMenuDrawItem) )
                {
                    var theContextRawPtr = UnsafeMutableRawPointer(bitPattern:0)

                    GetEventParameter(inEvent,
                                      EventParamName(kEventParamCGContextRef),
                                      EventParamType(typeCGContextRef),
                                      nil,
                                      MemoryLayout<CGContext>.size,
                                      nil,
                                      &theContextRawPtr)

                    let theContext: CGContext? = unsafeBitCast(theContextRawPtr, to: CGContext.self)

                    theContext!.saveGState()
                    let theNSContext: NSGraphicsContext  = NSGraphicsContext(cgContext: theContext!,
                                                                             flipped: false)
                    NSGraphicsContext.current = theNSContext

                    NSGraphicsContext.saveGraphicsState()

                    let  isSelected: Bool = theTrackingData.itemSelected == theItemIndex
                    let  isEnabled:  Bool = theItem!.isEnabled

                    var theItemRect: HIRect = NSZeroRect
                    GetEventParameter(inEvent,
                                      EventParamName(kEventParamMenuItemBounds),
                                      EventParamType(typeHIRect),
                                      nil,
                                      MemoryLayout<HIRect>.size,
                                      nil,
                                      &theItemRect)
                    
                    // メニューバーの高さの計算。見た目の修正の為に1Pxだけ補正を入れている。
                    let theRealMenuBarHeight: CGFloat = (NSApp.mainMenu?.menuBarHeight)!
                        + ( CGFloat(theTrackingData.virtualMenuBottom) - self.size.height ) - 1.0
                    
                    // ローカル座標に変換(?)。原点がメニューの右下から右上にする。
                    theItemRect.origin.y = -1 * theItemRect.origin.y
                        + CGFloat(theTrackingData.virtualMenuBottom) - theRealMenuBarHeight

                    theItem?.drawKeyEquivalent(with:theItemRect,
                                              context:theContext!,
                                              selected:isSelected,
                                              enabled:isEnabled)

                    // Restore Context
                    NSGraphicsContext.restoreGraphicsState()
                    theContext!.restoreGState()
                }
                else if( GetEventKind(inEvent) == OSType(kEventMenuMeasureItemWidth) )
                {
                    var theMenuItemWidth: Int16 = 0;

                    GetEventParameter(inEvent,
                                      EventParamName(kEventParamMenuItemWidth),
                                      EventParamType(typeSInt16),
                                      nil,
                                      MemoryLayout<Int16>.size,
                                      nil,
                                      &theMenuItemWidth)

                    theMenuItemWidth = theMenuItemWidth + Int16(theItem!.minimalKeyBindingAndMarginWidth)

                    SetEventParameter(inEvent,
                                      EventParamName(kEventParamMenuItemWidth),
                                      EventParamType(typeSInt16),
                                      MemoryLayout<Int16>.size,
                                      &theMenuItemWidth)
                }
            }
        }
        
        return  theResult
    }

    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    var eventHandlerRef: EventHandlerRef? = nil

    typealias GetCarbonMenuFuncType = @convention(c) (NSMenu) -> Menu?

    static var getCarbonMenu1: GetCarbonMenuFuncType? = nil
    static var getCarbonMenu2: GetCarbonMenuFuncType? = nil

    var  menuRef: Menu?
    {
        if( MINTLMenu.getCarbonMenu1 == nil && MINTLMenu.getCarbonMenu2 == nil )
        {
            MINTLMenu.getCarbonMenu1 = MINTLDynamicLink.symbolPtrFromSelf(symbol:"_NSGetCarbonMenu",
                                                                          type:GetCarbonMenuFuncType.self)
            
            MINTLMenu.getCarbonMenu2 = MINTLDynamicLink.symbolPtrFromSelf(symbol:"_NSGetCarbonMenu2",
                                                                          type:GetCarbonMenuFuncType.self)
        }
        
        if( MINTLMenu.getCarbonMenu1 != nil && MINTLMenu.getCarbonMenu2 != nil )
        {
            let theResult = MINTLMenu.getCarbonMenu1!(self)
            
            return theResult != nil ? theResult : MINTLMenu.getCarbonMenu2!(self)
        }
        else if( MINTLMenu.getCarbonMenu1 != nil )
        {
            return MINTLMenu.getCarbonMenu1!(self)
        }
        else if( MINTLMenu.getCarbonMenu2 != nil )
        {
            return MINTLMenu.getCarbonMenu2!(self)
        }
        
        return nil        
    }
    
    //        extern EventTargetRef GetMenuEventTarget(MenuRef); // HIToolbox/CarbonEvents.h 32bit Only
    typealias GetMenuEventTargetType = @convention(c) (Menu) -> EventTargetRef

    static var getMenuEventTarget: GetMenuEventTargetType? = nil
    
    func getMenuEventTarget(_ Menu: Menu) -> EventTargetRef
    {
        if ( MINTLMenu.getMenuEventTarget == nil )
        {
            MINTLMenu.getMenuEventTarget = MINTLDynamicLink.symbolPtrFromSelf(symbol:"GetMenuEventTarget",
                                                                          type:GetMenuEventTargetType.self)
        }
        
        return MINTLMenu.getMenuEventTarget!(Menu)
    }

    /*
    extern OSStatus       ChangeMenuItemAttributes(
                                                   MenuRef menu,
                                                   MenuItemIndex item,
                                                   MenuItemAttributes setTheseAttributes,
                                                   MenuItemAttributes clearTheseAttributes
                                                   );// HIToolbox/Menus.h 32bit Only
    */
    typealias ChangeMenuItemAttributesType =
        @convention(c) (Menu, MenuItemIndex, MenuItemAttributes, MenuItemAttributes) -> OSStatus
    
    static var changeMenuItemAttributes: ChangeMenuItemAttributesType? = nil
    
    func changeMenuItemAttributes(_ menu: Menu,
                                  _ item: MenuItemIndex,
                                  _ setTheseAttributes: MenuItemAttributes,
                                  _ clearTheseAttributes: MenuItemAttributes) -> OSStatus
    {
        if ( MINTLMenu.changeMenuItemAttributes == nil )
        {
            MINTLMenu.changeMenuItemAttributes =
                MINTLDynamicLink.symbolPtrFromSelf(symbol:"ChangeMenuItemAttributes",
                                                   type:ChangeMenuItemAttributesType.self)
        }
        
        return MINTLMenu.changeMenuItemAttributes!(menu, item, setTheseAttributes, clearTheseAttributes)
    }

    
    // MARK: - Utility methods
}


