//
//  MINTLMenuItem.swift
//  MenuTester
//
//  Created by narita on 2017/08/14.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

class MINTLMenuItem: NSMenuItem
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
        
    // MARK: - drawing methods
    
    func drawKeyEquivalent(with rect: NSRect, context: CGContext, selected: Bool, enabled: Bool)
    {
        let theRounded = NSBezierPath.init(roundedRect: self.roundedRectFromMenu(rect: rect),
                                           xRadius: 3,
                                           yRadius: 3)
        self.backgroundColor(with: selected, enabled: enabled).set()
        theRounded.fill()
        
        context.setAllowsFontSmoothing(false)

        (self.keyBindingString as NSString).draw(in: self.keyBindingStringDrawingRectFromMenu(rect: rect),
                                                 withAttributes: self.keyBindingStringAttributesWith(with:
                                                    self.foregroundColor(with: selected, enabled: enabled)))
                
        context.setAllowsFontSmoothing(true)
    }
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @IBInspectable var keyBindingString: String = ""
    
    // ショートカットキー文字列の画面上の幅
    var keyBindingWidth: CGFloat
    {        
         return self.keyBindingString.size(withAttributes: self.keyBindingStringAttributes ).width
    }
    // ショートカットキーの描画文字列の属性
    var keyBindingStringAttributes : Dictionary<NSAttributedStringKey, Any>
    {
        let theStyle = NSMutableParagraphStyle.init()
        theStyle.alignment = NSTextAlignment.right
        
        return [
            NSAttributedStringKey.paragraphStyle: theStyle,
            NSAttributedStringKey.font          : NSFont.init(name:"Courier", size:13)!]
    }

    // ショートカットキーの文字の色を含めた描画文字列の属性
    func keyBindingStringAttributesWith(with foregroundColor: NSColor) -> Dictionary<NSAttributedStringKey, Any>
    {
        var theDict: Dictionary<NSAttributedStringKey, Any> = self.keyBindingStringAttributes

        theDict[NSAttributedStringKey.foregroundColor] = foregroundColor

        return theDict
    }
    
    // ショートカット文字列の描画範囲の矩形
    func keyBindingStringDrawingRectFromMenu(rect: NSRect) -> NSRect
    {
        return
            NSMakeRect(
                NSMaxX(rect) - self.keyBindingWidth - self.minimalkTailMargin,
                NSMinY(rect) + 1,
                self.keyBindingWidth,
                rect.size.height - 2)
    }

    // ショートカットが置かれる場所のバックグラウンドの角丸の矩形
    func roundedRectFromMenu(rect: NSRect) -> NSRect
    {
        return
            NSMakeRect(
                 NSMaxX(rect) - self.keyBindingWidth - self.minimalkTailMargin - 2,
                 NSMinY(rect) + 1,
                 self.keyBindingWidth + 3, // 描画領域の幅を少しだけ広げる
                 rect.size.height - 2)
    }
    
    var minimalLeadMargin: CGFloat
    {
        return 40
    }
    
    var minimalkTailMargin: CGFloat
    {
        return 8
    }

    var minimalKeyBindingAndMarginWidth: CGFloat
    {
        return self.minimalLeadMargin + self.keyBindingWidth + self.minimalkTailMargin
    }

    // MARK: - Utility methods

    func foregroundColor(with selected: Bool, enabled: Bool) -> NSColor
    {
        return enabled  ?
                            (
                                selected ? NSColor.selectedMenuItemTextColor : NSColor.controlTextColor
                            )
                        :
                            NSColor.disabledControlTextColor
    }

    func backgroundColor(with selected: Bool, enabled: Bool) -> NSColor
    {
        return enabled  ?
                            (
                                selected ? NSColor.selectedControlColor : NSColor.secondarySelectedControlColor
                            )
                        :
                            NSColor.controlColor
    }
}
